package mako5.ctrl;


import static org.lwjgl.opengl.GL11.GL_DEPTH_TEST;
import static org.lwjgl.opengl.GL11.GL_LEQUAL;
import static org.lwjgl.opengl.GL11.GL_MODELVIEW;
import static org.lwjgl.opengl.GL11.GL_NICEST;
import static org.lwjgl.opengl.GL11.GL_PERSPECTIVE_CORRECTION_HINT;
import static org.lwjgl.opengl.GL11.GL_POINT_SMOOTH_HINT;
import static org.lwjgl.opengl.GL11.GL_PROJECTION;
import static org.lwjgl.opengl.GL11.GL_SMOOTH;
import static org.lwjgl.opengl.GL11.glClearColor;
import static org.lwjgl.opengl.GL11.glClearDepth;
import static org.lwjgl.opengl.GL11.glDepthFunc;
import static org.lwjgl.opengl.GL11.glEnable;
import static org.lwjgl.opengl.GL11.glHint;
import static org.lwjgl.opengl.GL11.glLoadIdentity;
import static org.lwjgl.opengl.GL11.glMatrixMode;
import static org.lwjgl.opengl.GL11.glShadeModel;
import static org.lwjgl.opengl.glu.GLU.gluPerspective;

import java.util.ArrayList;
import java.util.HashMap;

import org.lwjgl.LWJGLException;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;

/**
 * Controls a group of game states and is capable of running them.
 * 
 * @author Paul Jarrett
 * @version 28-Sept 2005
 * @version 12-Apr 2006
 * @version 16-Aug 2006
 */
public class GameStateMachine {

	/**
	 * The current game state.
	 * 
	 * @since 23-May 2006
	 */
	private GameState current;

	/**
	 * This is the list of all game states.
	 * 
	 * @since 23-May 2006
	 */
	private ArrayList<GameState> gameStates;

	/**
	 * Stored data within this game state machine.
	 * 
	 * @since 23-May 2006
	 */
	private HashMap data;

	/**
	 * This is true when the program should end. It being false continues the
	 * game's main loop.
	 * 
	 * @since 23-May 2006
	 */
	private boolean done;



	/**
	 * Creates a new GameStateMachine.
	 * 
	 * @since 23-May 2006
	 */
	public GameStateMachine() {
		current = null;
		gameStates = new ArrayList<GameState>();
		data = new HashMap();
	}



	/**
	 * Adds a new state.
	 * 
	 * @param s
	 *            the state to add
	 * @since 23-May 2006
	 */
	public void add(GameState s) {
		// only adds the state if it is not already in the list
		if (!gameStates.contains(s)) {

			gameStates.add(s);
			s.setMachine(this);

			// if is the first state added, then make it the current state
			if (gameStates.size() == 1) {
				current = gameStates.get(0);
			}
		}
	}



	/**
	 * Returns the current game state.
	 * 
	 * @return the current game state, or null if none has been assigned.
	 * @since 23-May 2006
	 */
	public GameState getCurrent() {
		return current;
	}



	/**
	 * Sets the current state.
	 * 
	 * @param i
	 *            the index of the game state to switch to
	 * 
	 * @since 28-Sept 2005
	 * @since 12-Apr 2006
	 */
	public void setCurrent(int i) {
		current.store();
		current.cleanup();
		current = gameStates.get(i);
		current.refresh();
	}



	/**
	 * Returns the stored game data for this.
	 * 
	 * @return a hashmap of stored game data
	 * @since 16-Jul 2006
	 */
	public HashMap getData() {
		return data;
	}



	/**
	 * Makes the window.
	 * 
	 * @return false if fails
	 */
	protected boolean createWindow() {
		try {
			Display.create();
			Display.setFullscreen(true);
			return true;
		} catch (LWJGLException e) {
			return false;
		}
	}



	// Initializes input devices
	protected boolean initInput() {
		try {
			Keyboard.create();
			Mouse.create();
			return true;
		} catch (LWJGLException e) {
			return false;
		}
	}



	// Initializes openGl with some default values
	protected boolean initGL() {
		// Use smooth shading
		glShadeModel(GL_SMOOTH);

		// Sets up the clear
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		glClearDepth(1.0);

		// Enable depth testing
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LEQUAL);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		// Calculate The Aspect Ratio Of The Window
		gluPerspective(45.0f, // distance (or angle) to view plate?
				1280.0f / 800.0f, // aspect ratio of the window
				0.1f, 10000.0f);
		glMatrixMode(GL_MODELVIEW); // Select The Modelview Matrix

		// Really Nice Perspective Calculations
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

		// Really Nice Point Smoothing
		glShadeModel(GL_SMOOTH);
		glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);

		return true;
	}



	/**
	 * Initializes all basic components.
	 * 
	 * Performs initialization in the following manner:
	 * <ul>
	 * <li>createWindow()</li>
	 * <li>initInput()</li>
	 * <li>initGL()</li>
	 * </ul>
	 * Kills the program if any portion fails
	 * 
	 * @since 16-Jul 2006
	 */
	protected void initAll() {
		boolean success = createWindow() && initInput() && initGL();

		if (!success) {
			cleanup();
			done = true;
			System.exit(0);
		}
	}



	/**
	 * Terminates the program.
	 * 
	 * @since 28-Sept 2005
	 * @since 12-Apr 2006
	 */
	public void exit() {
		// cleans up after the current state
		current.cleanup();

		// signals that the program wants to quit
		done = true;
	}



	/**
	 * Cleans up after the program has finished. Gets rid of the display, and
	 * mouse and keyboard input devices.
	 * 
	 * @since 16-Jul 2006
	 */
	public void cleanup() {
		Display.destroy();
		Keyboard.destroy();

		if (Mouse.isGrabbed()) {
			Mouse.setGrabbed(false);
		}
		Mouse.destroy();
	}



	// Loads the necessary states
	public void loadStates() {
	}



	// Loads other necessary resources
	public void loadResources() {
	}



	/**
	 * Runs the program.
	 * 
	 * This is how it does it:
	 * <ul>
	 * <li>Initialize components</li>
	 * <li>Load states</li>
	 * <li>Load game resources</li>
	 * <li>Main loop</li>
	 * <li>Cleanup</li>
	 * 
	 * @since 28-Sept 2005
	 * @since 12-Apr 2006
	 * @since 29-Apr 2006
	 */
	public void run() {
		// we're not done yet
		done = false;

		// Should setup opengl
		initAll();

		// Loads states and resources
		loadStates();
		loadResources();

		if ((current == null) && (gameStates.size() > 0)) {
			current = gameStates.get(0);
		}

		// the state should have at least one state
		assert (current != null);

		// Refreshes the current state
		current.refresh();

		// Counters to measure elapsed time
		long elapsed, last;
		last = elapsed = System.currentTimeMillis();

		// Main loop
		while (!done) {
			elapsed = System.currentTimeMillis() - last;
			last += elapsed;

			current.update(elapsed);
			current.render();
			Display.update();
		}

		// Cleanup after opengl
		cleanup();
	}
}