package mako5.evnt;


/**
 * The time keeper that knows the number of elapsed milliseconds, the number of
 * frames, and can calculate the time per frame.
 * 
 * Keeps a running mSPF (milliseconds/frame) count and can be used as a timer
 * function for smoothing of programs.
 * 
 * @author Paul Jarrett
 * @since 03-Apr 2006
 * @version 03-Apr 2006
 * @version 20-Apr 2006
 * @version 17-May 2006
 * @version 02-Jul 2006
 * @version 07-Jul 2006
 */

public class TimeGnome {

	/**
	 * The current time.
	 */
	private long now;

	/**
	 * The elapsed time since the last clocked time.
	 */
	private long elapsed;

	/**
	 * The number of frames.
	 */
	private long frames;

	/**
	 * The number of milliseconds per frame.
	 */
	private float spf;

	/**
	 * The number of frames before the gnome resets itself. When this value is <
	 * 1, then the gnome will never before reset.
	 */
	private long framesTilReset;



	/**
	 * Initializes values for all variables.
	 * 
	 * @since 03-Apr 2006
	 * @since 07-Jul 2006
	 */
	public TimeGnome() {
		now = 0L;
		elapsed = 0L;
		frames = 1L; // prevents divide by 0 exception
		spf = 0f;
		framesTilReset = -1L;
	}



	/**
	 * Initializes values for all variables.
	 * 
	 * @since 03-Apr 2006
	 * @since 07-Jul 2006
	 */
	public TimeGnome(long resetFrames) {
		now = 0L;
		elapsed = 0L;
		frames = 1L; // prevents divide by 0 exception
		spf = 0f;
		framesTilReset = resetFrames;
	}



	/**
	 * Provides a starting time for the gnome and resets the number of frames.
	 * This in turn lets the program determine when to start counting frames,
	 * and elapsed time.
	 * 
	 * @since 03-Apr 2006 (known as setup(void))
	 * @since 17-May 2006 (known as setup(void))
	 * @since 07-Jul 2006
	 */
	public void reset() {
		// gets the time just before now
		now = System.currentTimeMillis() - 1;

		// resets the number of frames
		// prevents a divide by 0 exception
		frames = 1L;

		elapsed = 0L;
	}



	/**
	 * Determines the mSPF. This information is found using the given elapsed
	 * time (in milliseconds) and number of frames.
	 * 
	 * @since 03-Apr 2006
	 * @since 17-May 2006
	 * @since 07-Jul 2006
	 */
	protected void calcMSPF() {
		spf = elapsed / frames;
	}



	/**
	 * Marks the current time, and finds the difference between this time and
	 * the previous time.
	 * 
	 * @since 03-Apr 2006
	 * @since 17-May 2006
	 */
	protected void clockNext() {
		// records the current time
		long current = System.currentTimeMillis();

		// finds the elapsed time
		elapsed += current - now;

		// updates the "now" time
		now = current;
	}



	/**
	 * Recalcs elapsed time and increments the number of frames.
	 * 
	 * @since 03-Apr 2006
	 */
	public void update() {
		frames++; // increments the frame count

		clockNext(); // resets the elapsed time

		calcMSPF(); // calculates the frames per second

		if (framesTilReset > 0) {
			if (frames > framesTilReset) {
				reset();
			}
		}
	}



	/**
	 * Returns the number of milliseconds per frame.
	 * 
	 * @return the number of milliseconds per frame
	 * 
	 * @since 03-Apr 2006
	 */
	public float getMSPF() {
		return spf;
	}
}