package mako5.gfx;


import org.lwjgl.opengl.GL11;

/**
 * A color that may be applied.
 * 
 * @author Paul Jarrett
 * @version 03-Feb 2006
 */

public final class Color implements Cloneable {

	public static final Color YELLOW = new Color(1f, 1f, 0f, 1f);

	// obey the domain [0.0,1.0]
	private float m_fRed; // red component

	private float m_fGreen; // green component

	private float m_fBlue; // blue component

	private float m_fAlpha; // alpha component



	/**
	 * Makes a new color.
	 */
	public Color(float fRed_, float fGreen_, float fBlue_, float fAlpha_) {
		m_fRed = fRed_;
		m_fBlue = fBlue_;
		m_fGreen = fGreen_;
		m_fAlpha = fAlpha_;

		// ensures bounds of [0.0f,1.0f] on components
		checkBounds();
	}



	/**
	 * Makes a new color.
	 * 
	 * Uses 1.0 as the alpha component.
	 */
	public Color(float fRed_, float fGreen_, float fBlue_) {
		this(fRed_, fGreen_, fBlue_, 1.0f);
	}



	/**
	 * Makes a copy of another color.
	 */
	public Color(Color c) {
		this(c.getRed(), c.getGreen(), c.getBlue(), c.getAlpha());
	}



	/**
	 * Clones this color.
	 */
	@Override
	public Object clone() {
		return new Color(getRed(), getGreen(), getBlue(), getAlpha());
	}



	/**
	 * Sets this as the current color for openGL.
	 */
	public void glApply() {
		GL11.glColor4f(m_fRed, m_fGreen, m_fBlue, m_fAlpha);
	}



	/**
	 * Changes the red component.
	 */
	public void setRed(float fRed_) {
		m_fRed = fRed_;
		checkBounds();
	}



	/**
	 * Changes the green component.
	 */
	public void setGreen(float fGreen_) {
		m_fGreen = fGreen_;
		checkBounds();
	}



	/**
	 * Changes the blue component.
	 */
	public void setBlue(float fBlue_) {
		m_fBlue = fBlue_;
		checkBounds();
	}



	/**
	 * Changes the alpha component.
	 */
	public void setAlpha(float fAlpha_) {
		m_fAlpha = fAlpha_;
		checkBounds();
	}



	/**
	 * Sets this color equal to another color.
	 * 
	 * @param rColor_
	 *            the target color to copy
	 */
	public void set(Color rColor_) {
		// ensures that the color is not null
		assert (rColor_ != null);

		setRed(rColor_.getRed());
		setGreen(rColor_.getGreen());
		setBlue(rColor_.getBlue());
		setAlpha(rColor_.getAlpha());
	}



	/**
	 * Sets the color components of this.
	 * 
	 * @since 11-Mar 2006
	 */
	public void set(float r, float g, float b, float a) {
		setRed(r);
		setGreen(g);
		setBlue(b);
		setAlpha(a);
		checkBounds();
	}



	/**
	 * Returns the red component.
	 * 
	 * @return the red component
	 */
	public float getRed() {
		return m_fRed;
	}



	/**
	 * Returns the green component.
	 * 
	 * @return the green component
	 */
	public float getGreen() {
		return m_fGreen;
	}



	/**
	 * Returns the blue component.
	 * 
	 * @return the blue component
	 */
	public float getBlue() {
		return m_fBlue;
	}



	/**
	 * Returns the alpha component.
	 * 
	 * @return the alpha component
	 */
	public float getAlpha() {
		return m_fAlpha;
	}



	/**
	 * Ensures all components obey [0.0,1.0].
	 * 
	 * @since 05-Mar 2006
	 */
	private final void checkBounds() {
		if (m_fRed < 0f) {
			m_fRed = 0.0f;
		} else if (m_fRed > 1f) {
			m_fRed = 1.0f;
		}
		if (m_fGreen < 0f) {
			m_fGreen = 0.0f;
		} else if (m_fGreen > 1f) {
			m_fGreen = 1.0f;
		}
		if (m_fBlue < 0f) {
			m_fBlue = 0.0f;
		} else if (m_fBlue > 1f) {
			m_fBlue = 1.0f;
		}
	}
}
