package mako5.gfx;


import java.nio.IntBuffer;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;

/**
 * An image designed for use with OpenGL. This is specifically for use in a 2D
 * environment in which sprites are needed. These types of images have their
 * image data stored as a reference in video memory like those in C++.
 * 
 * @author Paul Jarrett
 * @version 24-Jun 2006
 */

public class MaskedImage implements Image {

	/**
	 * Used to mark an image as unusable.
	 * 
	 * @since 24-Jul 2006
	 */
	protected static final int DESTROYED_ID = -1;

	/**
	 * The image storage reference for where the address of the texture. This is
	 * used to bind the image before it is used.
	 * 
	 * @since 24-Jul 2006
	 */
	private int dataRef;

	/**
	 * The image storage reference for where the address of the mask texture.
	 * This is used to bind the image before it is used.
	 * 
	 * @since 24-Jul 2006
	 */
	private int maskRef;

	/**
	 * The height of this image in pixels.
	 * 
	 * @since 24-Jul 2006
	 */
	private int height;

	/**
	 * The width of this image in pixels.
	 * 
	 * @since 24-Jul 2006
	 */
	private int width;



	/**
	 * Makes a new image with the given height, width, and texture id. Asserts
	 * that the height and width are not negative.
	 * 
	 * @since 24-Jul 2006
	 */
	public MaskedImage(int width_, int height_, int texID, int mskID) {
		// checks the preconditions
		assert (height_ > 0);
		assert (width_ > 0);

		// performs initializations
		height = height_;
		width = width_;
		dataRef = texID;
		maskRef = mskID;
	}



	/**
	 * Attempts to destroy this image. This can only be done when then image has
	 * video memory data to destroy. If it performs this task, then it will
	 * return the value of "true", otherwise returns "false".
	 * 
	 * @return true if succeeds, false if operation fails
	 * @since 24-Jul 2006
	 */
	public boolean destroy() {
		// if this is destroyed, then it cannot be done again
		if (isDestroyed()) {
			// operation failed
			return false;
		} else {
			// list of objects to destroy
			IntBuffer cleanupList;

			// adds this to the list of objects to destroy
			cleanupList = BufferUtils.createIntBuffer(1);
			cleanupList.put(dataRef).flip();

			// destroys the image information
			GL11.glDeleteTextures(cleanupList);

			// marks this image as destroyed
			dataRef = DESTROYED_ID;

			// indicates success
			return true;
		}
	}



	/**
	 * Returns true if this image has been destroyed and is therefore unusable.
	 * 
	 * @since 24-Jul 2006
	 */
	public boolean isDestroyed() {
		return dataRef == DESTROYED_ID;
	}



	/**
	 * Returns the width of this image.
	 * 
	 * @since 24-Jul 2006
	 */
	public int getWidth() {
		return width;
	}



	/**
	 * Returns the height of this image.
	 * 
	 * @since 24-Jul 2006
	 */
	public int getHeight() {
		return height;
	}



	/**
	 * This does not support resizing of images and simply returns false.
	 * 
	 * @since 24-Jul 2006
	 */
	public boolean resize(int width_, int height_) {
		return false;
	}



	/**
	 * Renders this image so that the bottom left corner is at the desired
	 * coordinates, and image data goes up and to the right. The texture data is
	 * bound during this operation, which can result in performance hits if the
	 * same image is used for many rendering calls consecutively. In order to
	 * regain performance, bind() may be called, followed by multiple
	 * blindRender() calls.
	 * 
	 * @param x
	 *            the target x-coordinate to render to
	 * @param y
	 *            the target y-coordinate to render to
	 * @since 04-Mar 2006
	 */
	public void render(float x, float y) {
		GL11.glEnable(GL11.GL_BLEND);

		GL11.glColor4f(1f, 1f, 1f, 1f);

		GL11.glBlendFunc(GL11.GL_DST_COLOR, GL11.GL_ZERO);

		// applies texturing data
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, maskRef);

		// renders this
		blindRender(x, y);

		GL11.glBlendFunc(GL11.GL_ONE, GL11.GL_ONE);

		// applies image texturing data
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, dataRef);

		// renders the actual image
		blindRender(x, y);

		GL11.glDisable(GL11.GL_BLEND);
	}



	/**
	 * Renders this image centered at the desired coordinates. This can be
	 * useful when you would rather store an object's location, and have the
	 * image be shown at its location and not have to make the correction
	 * yourself.
	 * 
	 * @param x
	 *            the target x-coordinate to render to
	 * @param y
	 *            the target y-coordinate to render to
	 * @since 04-Mar 2006
	 */
	public void renderAt(float x, float y) {
		// corrects rendering destination coordinates to center image
		x -= (width / 2);
		y -= (height / 2);

		render(x, y);

		/*
		 * //applies mask texturing data GL11.glBindTexture(GL11.GL_TEXTURE_2D,
		 * maskRef);
		 * 
		 * //corrects rendering destination coordinates to center image x -=
		 * (width/2); y -= (height/2);
		 * 
		 * //renders the mask blindRender(x,y);
		 * 
		 *  // applies image texturing data
		 * GL11.glBindTexture(GL11.GL_TEXTURE_2D, dataRef);
		 *  // renders the image blindRender(x,y);
		 */

	}



	/**
	 * Renders a quad of the image width and height without binding the texture
	 * first. This is done so that the bottom left corner is at the desired
	 * coordinates, and image data goes up and to the right.
	 * 
	 * This can increase performance when the same texture is used multiple
	 * times, thereby removing the time needed for the bind call.
	 * 
	 * @param x
	 *            the target x-coordinate to render to
	 * @param y
	 *            the target y-coordinate to render to
	 * 
	 * @since 04-Mar 2006
	 */
	public void blindRender(float x, float y) {
		// //adjusts the target rendering coordinate
		// y -= height;

		// renders this
		GL11.glBegin(GL11.GL_TRIANGLE_STRIP);
		GL11.glTexCoord2f(0.f, 0.f);
		GL11.glVertex2f(x, y);
		GL11.glTexCoord2f(0.f, 1.f);
		GL11.glVertex2f(x, y + height);
		GL11.glTexCoord2f(1.f, 0.f);
		GL11.glVertex2f(x + width, y);
		GL11.glTexCoord2f(1.f, 1.f);
		GL11.glVertex2f(x + width, y + height);
		GL11.glEnd();
	}



	/**
	 * Renders this image centered at the desired coordinates.
	 * 
	 * This can be useful when you would rather store an object's location, and
	 * have the image be shown at its location and not have to make the
	 * correction yourself.
	 * 
	 * @param x
	 *            the target x-coordinate to render to
	 * @param y
	 *            the target y-coordinate to render to
	 * @since 04-Mar 2006
	 */
	public void blindRenderAt(float x, float y) {
		// corrects rendering destination coordinates to center image
		x -= (width / 2);
		y -= (height / 2);

		// renders the image
		blindRender(x, y);
	}
}