package mako5.gfx.j2d;


import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

import javax.swing.ImageIcon;

import org.lwjgl.opengl.GL11;

/**
 * Used to perform specialized operations upon Java2D image types.
 * 
 * @author Paul Jarrett
 * @version 04-Mar 2006
 */

public final class JImageUtils {

	/**
	 * Indicates that this class cannot be instantiated.
	 */
	protected JImageUtils() {
	}



	/**
	 * Loads a Java image from the given string file.
	 * 
	 * @param rSource_
	 *            the location of the image data
	 * @return the image at the given location, or null if the file is corrupt
	 *         or nonexistent
	 */
	public static Image load(String rSource_) {
		// the image to return
		Image image = null;

		try {
			// tries to load the image
			image = load(new File(rSource_).toURL());
		} catch (MalformedURLException excp) {
			// prints the error
			excp.printStackTrace();
		}

		// returns whatever image is
		return image;
	}



	/**
	 * Loads a Java image from the given file.
	 * 
	 * @param rSource_
	 *            the location of the image data
	 * @return the image at the given location
	 */
	public static Image load(URL rSource_) {
		// loads the image using an ImageIcon
		ImageIcon icon = new ImageIcon(rSource_);

		// returns the image for this
		return icon.getImage();
	}



	/**
	 * Converts a standard Java image into a buffered image.
	 * 
	 * @param rImage_
	 *            the input image
	 * @param iType_
	 *            the type of buffered image to use
	 * @return a buffered image version of the input image
	 */
	public static BufferedImage convertToBufferedImage(Image rImage_, int iType_) {
		// gets teh image width and height
		final int WIDTH = rImage_.getWidth(null);
		final int HEIGHT = rImage_.getHeight(null);

		// makes the destination image
		BufferedImage bi = new BufferedImage(WIDTH, HEIGHT, iType_);

		// copies the image
		copyImage(rImage_, bi);

		// returns the copied image
		return bi;
	}



	/**
	 * Converts a standard Java image into the given buffered image.
	 * 
	 * @param rImage_
	 *            the input image
	 * @param rDest_
	 *            the destination image
	 * @return a buffered image version of the input image
	 */
	public static BufferedImage copyImage(Image rImage_, BufferedImage rDest_) {
		// gets the graphics to draw to the buffered image
		Graphics2D g = rDest_.createGraphics();

		// draws the input image to the destination
		g.drawImage(rImage_, 0, 0, null);

		// recovers resources
		g.dispose();
		rDest_.flush();

		// returns the destination image
		return rDest_;
	}



	/**
	 * Takes the input image and returns an OpenGL image reference to a copied
	 * image.
	 * 
	 * NOTE: Not written by me
	 */
	public static int convertToOpenGL(BufferedImage rImage_) {
		BufferedImage in = rImage_;

		// /////////////////////////////////
		// //
		// not my code here... //
		// //
		// /////////////////////////////////
		/*
		 * The following code is taken directly for the LWJGL example code. It
		 * takes a Java Image and converts it into an OpenGL texture. This is a
		 * very powerful feature as you can use this to generate textures on the
		 * fly out of anything.
		 */
		// Flip Image
		AffineTransform tx = AffineTransform.getScaleInstance(1, -1);
		tx.translate(0, -in.getHeight(null));
		AffineTransformOp op = new AffineTransformOp(tx,
				AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
		in = op.filter(in, null);

		// Put Image In Memory
		ByteBuffer scratch = ByteBuffer.allocateDirect(4 * in.getWidth()
				* in.getHeight());

		byte data[] = (byte[]) in.getRaster().getDataElements(0, 0,
				in.getWidth(), in.getHeight(), null);
		scratch.clear();
		scratch.put(data);
		scratch.rewind();

		// Create A IntBuffer For Image Address In Memory
		IntBuffer buf = ByteBuffer.allocateDirect(4).order(
				ByteOrder.nativeOrder()).asIntBuffer();
		
		GL11.glGenTextures(buf); // Create Texture In OpenGL

		GL11.glBindTexture(GL11.GL_TEXTURE_2D, buf.get(0));
		// Typical Texture Generation Using Data From The Image

		// Linear Filtering
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER,
				GL11.GL_LINEAR);
		// Linear Filtering
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER,
				GL11.GL_LINEAR);
		// Generate The Texture
		GL11
				.glTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_RGBA, in
						.getWidth(), in.getHeight(), 0, GL11.GL_RGBA,
						GL11.GL_UNSIGNED_BYTE, scratch);

		return buf.get(0); // Return Image Address In Memory
	}
}