package mako5.math;


/**
 * An angle in radians. Useful when dealing with polar coordinates, or
 * determining the direction of vectors.
 * 
 * @author Paul Jarrett
 * @version 31-Jan 2006
 * @version 19-Jul 2006
 */

public final class Angle implements Cloneable {

	// used for reducing this angle
	public static final float PI = (float) (Math.PI);

	public static final float TWO_PI = (PI * 2.0f);

	public static final float HALF_PI = (PI / 2.0f);

	private float m_fRadians; // the measurement of this angle



	/**
	 * Makes a new angle.
	 */
	public Angle(float fRadians_) {
		m_fRadians = fRadians_;
	}



	/**
	 * Clones this angle
	 */
	@Override
	public Object clone() {
		return new Angle(m_fRadians);
	}



	/**
	 * Returns the measurement of this.
	 * 
	 * @return the measurement of this in radians
	 */
	public float toRadians() {
		return m_fRadians;
	}



	/**
	 * Returns the measurement of this in radians.
	 * 
	 * @return the measurement of this in degrees
	 */
	public float toDegrees() {
		return (float) (Math.toDegrees(m_fRadians));
	}



	/**
	 * Sets the value of this angle in radians.
	 * 
	 * @since 05-Jul 2006
	 */
	public void setRadians(float f) {
		m_fRadians = f;
	}



	/**
	 * Adds another angle to this.
	 * 
	 * @param other_
	 *            another angle
	 */
	public void add(Angle other_) {
		m_fRadians += other_.toRadians();

		reduce();
	}



	/**
	 * Adds to this angle.
	 * 
	 * @param value
	 *            the amount to add
	 * @since 1-Jul 2006
	 */
	public void add(float value) {
		m_fRadians += value;
	}



	/**
	 * Adds degrees to this.
	 */
	public void addDegrees(float fAmount_) {
		m_fRadians += (float) (Math.toRadians(fAmount_));
		reduce();
	}



	/**
	 * Adds radians to this.
	 * 
	 */
	public void addRadians(float fAmount_) {
		m_fRadians += fAmount_;
		reduce();
	}



	/**
	 * Subtracts another angle from this.
	 * 
	 */
	public void sub(Angle other_) {
		m_fRadians -= other_.toRadians();
		reduce();
	}



	/**
	 * Makes this angle equal -angle.
	 */
	public void flip() {
		m_fRadians = -m_fRadians;
	}



	/**
	 * Returns the sine of this in radians.
	 */
	public float sin() {
		return (float) (Math.sin(m_fRadians));
	}



	/**
	 * Returns the cosine of this in radians
	 */
	public float cos() {
		return (float) (Math.cos(m_fRadians));
	}



	/**
	 * Returns the tangent of this in radians
	 */
	public float tan() {
		return (float) (Math.tan(m_fRadians));
	}



	/**
	 * Reduces this angle by a single increment.
	 */
	private void reduce() {
		if (m_fRadians > TWO_PI) {
			m_fRadians -= TWO_PI;
		} else {
			if (m_fRadians < 0) {
				m_fRadians += TWO_PI;
			}
		}
	}



	/**
	 * Returns "Angle: " + ..value.. + " radians".
	 */
	@Override
	public String toString() {
		return "Angle:" + m_fRadians + " radians";
	}



	/**
	 * Determines an angle by using atan
	 */
	public static Angle atan(float x, float y) {
		// special cases!!
		// checks 90 and 270 conditions
		if (x == 0) {
			if (y < 0) {
				return new Angle((float) Math.toRadians(270));
			} else {
				return new Angle((float) Math.toRadians(90));
			}
		}

		// other calculations if necessary
		float rad = (float) (Math.atan(y / x));

		// boolean xpos = x > 0; //is x positive?
		// boolean ypos = y > 0; //is y positive?

		if (x < 0) {
			// quadrants II and III
			return new Angle(PI + rad);
		} else {
			// in quadrant IV
			if (y < 0) {
				return new Angle(PI + PI + rad);
			}
		}

		/*
		 * if( xpos ) { //angle is in quadrant I if( ypos ) { return new
		 * Angle(rad); } else { //angle is in quadrant IV return new Angle(PI +
		 * PI + rad); } } else { //angle is in quadrant II if( ypos ) { return
		 * new Angle(HALF_PI + rad); } else { //angle is in quadrant III return
		 * new Angle(PI + rad); } } }
		 */
		// quadrant I
		return new Angle(rad);
	}



	/**
	 * Performs an openGL rotation based upon this angle.
	 */
	public static void glRotate(Angle rAngle_, float x, float y, float z) {
		org.lwjgl.opengl.GL11.glRotatef(rAngle_.toDegrees(), x, y, z);
	}



	/**
	 * Performs an openGL rotation that is -angle.
	 */
	public static void glNegRotate(Angle rAngle_, float x, float y, float z) {
		org.lwjgl.opengl.GL11.glRotatef(-rAngle_.toDegrees(), x, y, z);
	}
}