package mako5.math;


import org.lwjgl.opengl.GL11;

/**
 * Represents a two-dimensional vector (R^2 vector).
 * 
 * 
 * <h2>Goals</h2>
 * <p>
 * The R2v class represents a vector in R^2 space. Applications include
 * representing points, vectors, and texture coordinates. Float values store the
 * coordinates providing floating point values while at a higher speed than
 * doubles. Strict floating point is not used to increase float accuracy.
 * </p>
 * 
 * 
 * <h2>Risks and Payoffs</h2>
 * <p>
 * Using this class results in object overhead that must be deleted using the
 * garbage collector. However, dot products and other vector calculations are
 * already set and tested. Returning both x- and y- coordinates at the same time
 * is also a plus.
 * </p>
 * 
 * 
 * 
 * 
 * <h2>Known problems</h2>
 * <p>
 * Using many of these as temporary objects will result in increased garbage
 * collection. The quickest solution to this is to use either an array-based
 * stack or queue to allow to O(1) insertion and removal of global scratch
 * objects. The operation looks similar to this:
 * 
 * <pre>
 * //declared ahead of time
 * Stack q = new ArrayStack(12);
 * 
 * for (int i = 0; i &lt; q.capacity(); q++)
 * 	stack.push(new R2v());
 * //...
 * //in method
 * R2v temp = stack.pop();
 * temp.set(0, 0); //or other staring temp value
 * //use temp
 * stack.push(temp);
 * temp = null; //not necessarily required
 * </pre>
 * 
 * This keeps the number of objects constant and the temporary references used
 * won't cause as severe a memory problem. The speed of the operations ensure
 * that a signifant decrease in performance does not occur.
 * </p>
 * 
 * 
 * 
 * @author Paul Jarrett
 * @version 29-Sept-2005
 * @version 07-Apr 2006
 * @version 1-Jul 2006
 */

public final class R2v implements Cloneable {

	private float x, y;



	/**
	 * Creates a new R2v.
	 * 
	 * @param x_
	 *            the x-coordinate
	 * @param y_
	 *            the y-coordinate
	 */
	public R2v(float x_, float y_) {
		this.x = x_;
		this.y = y_;
	}



	/**
	 * Creates a new R2v.
	 */
	public R2v(R2v v) {
		this(v.x, v.y);
	}



	/**
	 * Creates a new R2v at (0.0f,0.0f).
	 */
	public R2v() {
		this(0.0f, 0.0f);
	}



	/**
	 * Clones this R2v.
	 */
	@Override
	public Object clone() {
		return new R2v(x, y);
	}



	/**
	 * Returns the x coordinate.
	 */
	public float getX() {
		return x;
	}



	/**
	 * Returns the y-coordinate.
	 */
	public float getY() {
		return y;
	}



	/**
	 * Sets this to another vector.
	 */
	public void set(R2v v) {
		x = v.x;
		y = v.y;
	}



	/**
	 * Sets this to the given x and y values.
	 */
	public void set(float x, float y) {
		this.x = x;
		this.y = y;
	}



	/**
	 * Sets the x value.
	 */
	public void setX(float x) {
		this.x = x;
	}



	/**
	 * Sets the y-value.
	 */
	public void setY(float y) {
		this.y = y;
	}



	/**
	 * Adds x and y to this.
	 */
	public void add(float x, float y) {
		this.x += x;
		this.y += y;
	}



	/**
	 * Adds a vector to this.
	 */
	public void add(R2v v) {
		this.x += v.x;
		this.y += v.y;
	}



	/**
	 * Subtracts from this.
	 */
	public void sub(float x, float y) {
		this.x -= x;
		this.y -= y;
	}



	/**
	 * Subtracts a vector from this.
	 */
	public void sub(R2v v) {
		this.x -= v.x;
		this.y -= v.y;
	}



	/**
	 * Multiplies this by a scalar.
	 */
	public void multi(float s) {
		this.x *= s;
		this.y *= s;
	}



	/**
	 * Divides this by a scalar.
	 */
	public void div(float s) {
		this.x /= s;
		this.y /= s;
	}



	/**
	 * Normalizes this vector.
	 */
	public void normalize() {
		float len = lengthSquared();

		// do nothing if has no length or is already a unit vector
		if ((len == 1f) || (len == 0f)) {
			return;
		} else {
			len = (float) (Math.sqrt(len));
			x /= len;
			y /= len;
		}
	}



	/**
	 * Returns the distance to another point.
	 */
	public float findDistanceTo(R2v v) {
		float a = v.x - x;
		float b = v.y - y;
		return (float) Math.sqrt((a * a) + (b * b));
	}



	/**
	 * Returns the angle of radians between this and another vector.
	 * 
	 * <pre>
	 *  Method:
	 *  cos(theta) = this * v / (this.length() * v.length());
	 *  acos( [this*v] / [length()*v.length()] );
	 * </pre>
	 * 
	 * @since 29-Sept-2005
	 */
	public float angleWith(R2v v) {
		float a;
		float b;

		a = dot(v);
		b = length() + v.length();

		return (float) Math.acos(a / b);
	}



	/**
	 * Returns the length squared of this vector.
	 */
	public float lengthSquared() {
		return x * x + y * y;
	}



	/**
	 * Returns the length of this vector.
	 */
	public float length() {
		return (float) Math.sqrt(lengthSquared());
	}



	/**
	 * Returns true if this is a unit vector.
	 * 
	 * @since 29-Sept-2005
	 */
	public boolean hasHat() {
		return length() == 1.0f;
	}



	/**
	 * Keeps the direction but sets a new length for this vector.
	 */
	public void setLength(float f) {
		float a = f / length();
		multi(a);
	}



	/**
	 * Takes the dot product of this and another vector.
	 */
	public float dot(R2v v) {
		return this.x * v.x + this.y * v.y;
	}



	/**
	 * Returns the angle that this makes with the positive x-axis.
	 */
	public Angle getDirection() {
		return Angle.atan(x, y);
	}



	/**
	 * Converts this vector into a polar value.
	 * 
	 * @return the polar equivalent of this vector
	 * @since 1-Jul 2006
	 */
	public Polar toPolar() {
		return new Polar(length(), getDirection().toRadians());
	}



	/**
	 * Converts this to a string
	 */
	@Override
	public String toString() {
		return "<" + x + "," + y + ">";
	}



	/**
	 * Sets this as an OpenGL 2D texture coordinate.
	 */
	public void glTexCoord() {
		GL11.glTexCoord2f(x, y);
	}



	/**
	 * Translates this in the x-y plane in openGL.
	 */
	public void glTranslate() {
		GL11.glTranslatef(x, y, 0.0f);
	}



	/**
	 * Sets this as a 2D vertex.
	 * 
	 * @since 05-Apr 2006
	 */
	public void glVertex() {
		GL11.glVertex2f(x, y);
	}
}