package mako5.math.phys;


import mako5.math.R2v;

/**
 * A horizontal or verticle boundary that cannot be passed.
 * 
 * @author Paul Jarrett
 * @version 18-Feb 2006
 */

public class Boundary extends Bounds implements Bounded {

	public static final boolean RIGHT = true;

	public static final boolean LEFT = false;

	public static final boolean ABOVE = true;

	public static final boolean BELOW = false;

	/*
	 * Wall types
	 */
	public static final boolean HORIZONTAL = true;

	public static final boolean VERTICAL = false;

	private boolean type; // the type of boundary that this is

	private boolean solidSide; // which side cannot be passed



	/**
	 * Makes a new Boundary.
	 * 
	 * @param bType_
	 *            whether this boundary is HORIZONTAL or VERTICAL
	 * @param bWallSide_
	 *            where the wall (solid) segment is located
	 * @param fLoc_
	 *            the location value of this wall
	 */
	public Boundary(final boolean bType_, final boolean bWallSide_, float fLoc_) {
		super(Bounds.BOUNDARY);

		type = bType_;
		solidSide = bWallSide_;

		if (type == HORIZONTAL) {
			aliasLoc(new R2v(0f, fLoc_));
		} else {
			aliasLoc(new R2v(fLoc_, 0f));
		}
	}



	/**
	 * Returns this.
	 * 
	 * @return this boundary object
	 * @since 01-Apr 2006
	 */
	public Bounds getBounds() {
		return this;
	}



	/**
	 * Returns true if the object broke this boundary...
	 * 
	 * not implemented.
	 * 
	 */
	@Override
	public boolean isCollision(R2v r) {
		return false;
	}



	/**
	 * Returns the type of boundary that this is.
	 * 
	 * @return the type of boundary that this is
	 * @since 01-Apr 2006
	 */
	public boolean getWallType() {
		return type;
	}



	/**
	 * Returns the solid side of the boundary
	 * 
	 * @return the solid side of the boundary
	 */
	public boolean getSolidSide() {
		return solidSide;
	}



	/**
	 * Returns true if the given object breaks this boundary.
	 */
	public boolean isCollision(Bounds rBounds_) {
		// was a collision detected?
		boolean change = false;

		if (rBounds_ instanceof BoundingCircle) {
			BoundingCircle circle = (BoundingCircle) rBounds_;
			R2v cLoc = circle.getLocation();

			// horizontal wall
			if (type == HORIZONTAL) {
				if (solidSide == ABOVE) {
					if (cLoc.getY() + circle.getRadius() > getLocation().getY()) {
						cLoc.setY(getLocation().getY() - circle.getRadius());
						change = true;
					}
				} else {
					if (cLoc.getY() - circle.getRadius() < getLocation().getY()) {
						cLoc.setY(getLocation().getY() + circle.getRadius());
						change = true;
					}
				}
			}
			// vertical wall
			else {
				if (solidSide == LEFT) {
					if (cLoc.getX() - circle.getRadius() < getLocation().getX()) {
						cLoc.setX(getLocation().getX() + circle.getRadius());
						change = true;
					}
				} else {
					if (cLoc.getX() + circle.getRadius() > getLocation().getX()) {
						cLoc.setX(getLocation().getX() - circle.getRadius());
						change = true;
					}
				}
			}
		}
		return change;
	}



	/**
	 * Changes the direction of a mass appropriately if it collides with this.
	 * 
	 * 
	 * @param rMass_
	 *            the mass to check
	 * 
	 * public boolean collide(Mass2D rMass_) { if(rMass_.getBounds() instanceof
	 * BoundingCircle) { BoundingCircle circle; R2v cLoc;
	 * 
	 * circle = (BoundingCircle)(rMass_.getBounds()); cLoc =
	 * circle.getLocation();
	 * 
	 * 
	 * //tests for each type of boundary if(type == HORIZONTAL) { if(solidSide ==
	 * ABOVE) { if(cLoc.getY()+circle.getRadius() > getLocation().getY()) {
	 * rMass_.getLocation().setY(getLocation().getY()-circle.getRadius());
	 * rMass_.getVelocity().setY(-rMass_.getVelocity().getY()); return true; } }
	 * else //solidSide == BELOW { if(cLoc.getY()-circle.getRadius() <
	 * getLocation().getY()) {
	 * rMass_.getLocation().setY(getLocation().getY()+circle.getRadius());
	 * rMass_.getVelocity().setY(-rMass_.getVelocity().getY()); return true; } }
	 *  } else //type == VERTICAL { if(solidSide == LEFT) {
	 * if(cLoc.getX()-circle.getRadius() <= getLocation().getX()) {
	 * rMass_.getLocation().setX(getLocation().getX()+circle.getRadius());
	 * rMass_.getVelocity().setX(-rMass_.getVelocity().getX()); return true; } }
	 * else //solidSide == RIGHT { if(cLoc.getX()+circle.getRadius() >=
	 * getLocation().getX()) {
	 * rMass_.getLocation().setX(getLocation().getX()-circle.getRadius());
	 * rMass_.getVelocity().setX(-rMass_.getVelocity().getX()); return true; } } } }
	 * return false; }
	 * 
	 */

	/**
	 * Do nothing on collision.
	 * 
	 * @param other
	 *            the other colliding object
	 */
	public void notifyCollision(Bounded other) {
	}
}