package mako5.math.phys;


import mako5.util.ArrayStack;
import mako5.math.*;

/**
 * Performs physics operations when requested.
 * 
 * @author Paul Jarrett
 * @version 05-Mar 2006
 */

public class Physics {

	private static ArrayStack<R2v> TEMP_VECTORS; // scratch vectors that can
													// be used

	// static initializer
	static {
		TEMP_VECTORS = new ArrayStack<R2v>(16);
		while (TEMP_VECTORS.size() < 16) {
			TEMP_VECTORS.push(new R2v());
		}
	}



	/**
	 * Applies a velocity vector to the given position vector with no
	 * acceleration.
	 * 
	 * Note that if lElapsed_ <= 0, then the minimum time step of 16L will be
	 * used.
	 * 
	 * @param rLoc_
	 *            the location vector
	 * @param rVel_
	 *            the velocity vector
	 * @param elapsed_
	 *            the elapsed time
	 * @since 05-Mar 2006
	 */
	public static void applyVelocity(R2v rLoc_, R2v rVel_, float elapsed_) {
		/*
		 * //applies a minimum time step if(lElapsed_ <= 0) { lElapsed_ = 16L; }
		 */

		// finds the difference in position
		R2v rDLoc = new R2v(rVel_);
		rDLoc.multi(elapsed_);

		// adds the difference in position
		rLoc_.add(rDLoc);
	}



	/**
	 * Applies a velocity vector to the given position vector with a constant
	 * acceleration vector. Updates the velocity and location.
	 * 
	 * Note that if lElapsed_ <= 0, then the minimum time step of 16L will be
	 * used.
	 * 
	 * @param rLoc_
	 *            the location vector
	 * @param rVel_
	 *            the velocity vector
	 * @param rAcc_
	 *            the acceleration vector
	 * @param elapsed_
	 *            the elapsed time
	 * @since 05-Mar 2006
	 * @deprecated 1-Jul 2006
	 */
	@Deprecated
	public static void applyVelocity(R2v rLoc_, R2v rVel_, R2v rAcc_,
			float elapsed_) {
		// the equation that we will be using
		/*
		 * Xf = Xi + Vi * t + (1/2)*a*t*t
		 */

		R2v initVel; // the initial velocity vector
		R2v accel; // the acceleration vector component

		// gets scratch vectors
		initVel = TEMP_VECTORS.pop();
		accel = TEMP_VECTORS.pop();

		initVel.set(rVel_);
		// initVel = new R2v(rVel_); //sets the initial velocity
		initVel.multi(elapsed_); // multiplies by the elapsed time

		accel.set(rAcc_);
		// accel = new R2v(rAcc_); //sets the acceleration
		accel.multi(0.5f * elapsed_ * elapsed_); // modiifes based upon
													// 0.5*t*t

		// System.out.print(rLoc_);

		// adds the two vectors to the location vector
		rLoc_.add(initVel);
		rLoc_.add(accel);

		accel.set(rAcc_);
		accel.multi(elapsed_);
		// accel.div(elapsed_*0.5f);

		rVel_.add(accel);

		// returns scratch vectors
		TEMP_VECTORS.push(initVel);
		TEMP_VECTORS.push(accel);
	}



	/**
	 * Applies a velocity vector to the given position vector with a constant
	 * acceleration vector. Updates the velocity and location.
	 * 
	 * Note that if lElapsed_ <= 0, then the minimum time step of 16L will be
	 * used.
	 * 
	 * @param rLoc_
	 *            the location vector
	 * @param rVel_
	 *            the velocity vector
	 * @param rAcc_
	 *            the acceleration vector
	 * @param elapsed_
	 *            the elapsed time
	 * @since 05-Mar 2006
	 * @deprecated 1-Jul 2006
	 */
	@Deprecated
	public static void applyVelocity(R2v rLoc_, Polar rVel_, R2v rAcc_,
			float elapsed_) {
		// the equation that we will be using
		/*
		 * Xf = Xi + Vi * t + (1/2)*a*t*t
		 */

		R2v initVel; // the initial velocity vector
		R2v accel; // the acceleration vector component

		// gets scratch vectors
		initVel = TEMP_VECTORS.pop();
		accel = TEMP_VECTORS.pop();

		initVel.set(rVel_.toVector());
		// initVel = new R2v(rVel_); //sets the initial velocity
		initVel.multi(elapsed_); // multiplies by the elapsed time

		accel.set(rAcc_);
		// accel = new R2v(rAcc_); //sets the acceleration
		accel.multi(0.5f * elapsed_ * elapsed_); // modiifes based upon
													// 0.5*t*t

		// System.out.print(rLoc_);

		// adds the two vectors to the location vector
		rLoc_.add(initVel);
		rLoc_.add(accel);

		accel.set(rAcc_);
		accel.multi(elapsed_);
		// accel.div(elapsed_*0.5f);

		R2v tempVelocity = (rVel_.toVector());
		tempVelocity.add(accel);

		// returns scratch vectors
		TEMP_VECTORS.push(initVel);
		TEMP_VECTORS.push(accel);
	}
}