package mako5.util;


/**
 * A circular array queue class.
 * 
 * @author Paul Jarrett
 * @version 04-Apr 2006
 * @see Queue
 */

public class ArrayQueue<T> implements Queue<T> {

	private static final int DEFAULT_CAPACITY = 100;

	private T[] contents; // the contents of the queue

	private int front; // next element to remove

	private int end; // where to add the next element

	private int count; // how many elements we have



	/**
	 * Makes an empty circular queue of size 0.
	 */
	public ArrayQueue() {
		contents = (T[]) (new Object[DEFAULT_CAPACITY]);
		front = 0;
		end = 0;
		count = 0;
	}



	/**
	 * Creates an empty queue using the specified capacity.
	 * 
	 * @param initialCapacity
	 *            indicates the initial size of the internal array.
	 */
	public ArrayQueue(int initialCapacity) {
		assert (initialCapacity > 0);
		contents = (T[]) (new Object[initialCapacity]);
		front = 0;
		end = 0;
		count = 0;
	}



	/**
	 * Adds an item to the queue.
	 * 
	 * @param val
	 *            the thing to add
	 */
	public void enqueue(T val) {
		assert (val != null);

		// queue is full so expand the capacity
		if (isFull()) {
			expandCapacity();
		}

		// set the element
		contents[end] = val;

		// increment the end of the queue
		end = increment(end);

		count++;
	}



	/**
	 * Removes the element at the front of this queue and returns a reference to
	 * it.
	 * 
	 * @return the element removed from the front of the queue
	 */
	public T dequeue() {
		assert (!isEmpty());

		// gets the front, deletes the data, and increments front
		T val = contents[front];
		contents[front] = null;
		front = increment(front);

		count--;

		return val;
	}



	/**
	 * Peeks at the first element in this queue.
	 * 
	 * @return the first element in this queue without dequeueing it
	 */
	public T first() {
		assert (!isEmpty());

		return contents[front];
	}



	/**
	 * Returns true when the queue is empty.
	 * 
	 * @return true when the queue is empty
	 */
	public boolean isEmpty() {
		return (!isFull() && (front == end));
	}



	/**
	 * Returns the current number of elements in the queue.
	 * 
	 * @return how many elements are currently in the queue
	 */
	public int size() {
		return count;
	}



	/**
	 * Increments the given value.
	 * 
	 * @param val
	 *            the value to increase
	 * @return the incremented value
	 */
	private int increment(int val) {
		return (val + 1) % (contents.length);
	}



	/**
	 * Returns true if this queue is full.
	 * 
	 * @return true if this queue is full
	 */
	public boolean isFull() {
		return (count == contents.length);
	}



	/**
	 * Clears all elements from this queue.
	 * 
	 * @since 05-Apr 2006
	 */
	public void clear() {
		for (int i = 0; i < contents.length; i++) {
			contents[i] = null;
		}
		count = 0;
		front = 0;
		end = 0;
	}



	/**
	 * Creates a new array to store the contents of this queue with twice the
	 * capacity of the old one. This is a circular representation so this method
	 * should be careful on how to expand the queue.
	 */
	private void expandCapacity() {

		T[] array = (T[]) (new Object[contents.length * 2]);

		for (int i = 0; i < count; i++) {
			array[i] = contents[i];
		}
		for (int j = count; j < array.length; j++) {
			array[j] = null;
		}

		contents = null;
		contents = array;
		end = count;// decrement(end);
		// incorrect placement of elements using the following
		// System.arraycopy(contents, 0, array, 0, contents.length);
	}



	/**
	 * Returns this as a string.
	 * 
	 * @return the string version of this
	 */
	@Override
	public String toString() {
		StringBuffer str = new StringBuffer();
		T val;

		for (int i = 0; i < count; i++) {
			val = dequeue();
			str.append("" + val + '\n');
			enqueue(val);
		}

		return str.toString();
	}
}