package mako5.util;


import java.io.PrintStream;

/**
 * 
 * <h2>Goals</h2>
 * <p>
 * The need for a global logging system was seen. The output can be redirected
 * to multiple locations given the type of message required. At the same time
 * all messages can be sent to the same source if desired and setup properly.
 * </p>
 * 
 * 
 * <h2>Risks and Payoffs</h2>
 * <p>
 * System.out.println(...) will no longer be used as a result, and the logs will
 * be able to be used for End-user problem identification if needed.
 * </p>
 * 
 * 
 * @author Paul Jarrett
 * @since 09-Apr 2006
 * @version 09-Apr 2006
 * @version 29-Apr 2006
 * @version 29-Jul 2006
 */
// TODO fix stack trace printing

public final class Log {

	/**
	 * Bit flag used to indicate something pertaining to an error message.
	 * 
	 * @since 29-Jul 2006
	 */
	public static final int ERROR = 1;

	/**
	 * Bit flag used to indicate something pertaining to an output message.
	 * 
	 * @since 29-Jul 2006
	 */
	public static final int OUT = 2;

	/**
	 * Bit flag used to indicate something pertaining to a debug-only message.
	 * 
	 * @since 29-Jul 2006
	 */
	public static final int DEBUG = 4;

	/**
	 * The current "Log" singleton instance.
	 * 
	 * @since 09-Apr 2006
	 */
	private static Log INSTANCE = null;

	/**
	 * Used to print information messages.
	 * 
	 * @since 09-Apr 2006
	 */
	private PrintStream out;

	/**
	 * The error print stream.
	 * 
	 * @since 09-Apr 2006
	 */
	private PrintStream err;

	/**
	 * The stream used for debug messages.
	 * 
	 * @since 29-Jul 2006
	 */
	private PrintStream debug;

	/**
	 * Indicates what flags are being held.
	 * 
	 * @since 29-Jul 2006
	 */
	private int allowedMessages;

	/**
	 * Contains flags that tell the log to print a stack trace when these
	 * message types are encountered. By default only does this for error
	 * messages.
	 * 
	 * @since 29-Jul 2006
	 */
	private int traceMessages;
	
	
	/**
	 * The stack of message streams which the next message will be sent to.
	 * @since 29-Jul 2006
	 */
	private ArrayStack<PrintStream> printStack;



	/**
	 * Makes a new Log.
	 * 
	 * Sets the output and error streams to System.out.
	 * 
	 * @since 29-Apr 2006
	 */
	private Log() {
		out = System.out;
		err = System.out;
		debug = System.out;
		
		allowedMessages = ERROR | OUT | DEBUG;
		traceMessages = ERROR;
		
		printStack = new ArrayStack<PrintStream>(3);
	}



	/**
	 * Cleans up when this log is finished.
	 * 
	 * @since 29-Apr 2006
	 */
	private void cleanup() {
		out.close();
		err.close();
		debug.close();
	}



	/**
	 * Called when this object is destroyed.
	 * 
	 * @since 29-Apr 2006
	 */
	@Override
	protected void finalize() {
		cleanup();
	}


	
	/**
	 * Returns true if the flag is contained in the int.
	 * @param flags_ an int that is the flags
	 * @param target_ the flag we are looking for
	 * @since 29-Jul 2006
	 */
	private static boolean hasFlag(int flags_, int target_)
	{
		return (flags_ & target_)  == target_;
	}
	

	/**
	 * Ensures that there is a singleton instance of this. If none exists, then
	 * one will be created.
	 * 
	 * @since 29-Apr 2006
	 */
	private static void checkInstance() {
		if (INSTANCE == null) {
			INSTANCE = new Log();
		}
	}



	/**
	 * Sets the requested streams. A stream is given, along with flags for which
	 * streams to set. This permits the setting of multiple streams with a
	 * single command.
	 * 
	 * @param stream_
	 *            the new stream_
	 * @param flags_
	 *            the streams to set
	 * @since 29-Apr 2006
	 */
	public static void setStream(PrintStream stream_, int flags_) {
		checkInstance();

		// checks the error flag
		if ((flags_ & ERROR) == ERROR) {
			// closes the previous stream
			INSTANCE.err.close();

			INSTANCE.err = stream_;
		}

		// checks the regular output flag
		if ((flags_ & OUT) == OUT) {
			// closes the previous stream
			INSTANCE.out.close();

			INSTANCE.out = stream_;
		}

		// checks the debug flag
		if ((flags_ & DEBUG) == DEBUG) {
			// closes the previous stream
			INSTANCE.debug.close();

			INSTANCE.debug = stream_;
		}
	}





	/**
	 * Prints a stack trace to the requested targets.
	 * 
	 * @param targets_ the flags for requested streams
	 * 
	 * @since 29-Apr 2006
	 * @since 29-Jul 2006
	 */
	public static void stackTrace(int targets_) {
		checkInstance();
		
		INSTANCE.populatePrintStack(targets_);
		
		while(!INSTANCE.printStack.isEmpty())
		{
			INSTANCE.printStack.pop().println(new Exception().getStackTrace());
		}
	}

	
	/**
	 * Returns the stream associated with the given flag.
	 * @param flag_ the stream we're looking for
	 * @since 29-Jul 2006
	 */
	private PrintStream getStream(int flag_)
	{		
		switch(flag_)
		{
			case ERROR:
				return err;
			case OUT:
				return out;
			case DEBUG:
				return debug;			
			default:
				// no stream defined for given flag
				return null;
		}
	}
	
	/**
	 * Given a print stream, returns the appropriate flag
	 * @param stream_ the target stream to associate
	 * @return the available flag, or 0 if it is not recognized
	 * @since 29-Jul 2006
	 */
	private int getFlagName(PrintStream stream_)
	{
		if(stream_ == err) return ERROR;
		if(stream_ == out) return OUT;
		if(stream_ == debug) return DEBUG;
		
		// not recognized
		return 0;
	}

	/**
	 * Adds the appropriate print streams to the stack as they
	 * are provided in the flags.
	 * 
	 * @params targets_ what to add to the print stack (ERROR,OUT,DEBUG)
	 * @since 29-Jul 2006
	 */
	private void populatePrintStack(int targets_)
	{		
		if(hasFlag(targets_,ERROR) && hasFlag(allowedMessages,ERROR)){
			printStack.push(err);
		}
		
		if(hasFlag(targets_,OUT) && hasFlag(allowedMessages,OUT)){
			printStack.push(out);
		}
		
		if(hasFlag(targets_,DEBUG) && hasFlag(allowedMessages,DEBUG)){
			printStack.push(debug);
		}	
	}

	/**
	 * Prints something to the given streams.
	 * 
	 * @param message_
	 *            what to print
	 * @param targets_
	 *            the target stream(s)
	 * @since 29-Jul 2006
	 */
	public static void println(String message_, int targets_) {
		checkInstance();
		
		// adds the appropriate strings to the print stack
		INSTANCE.populatePrintStack(targets_);
		
		while(!INSTANCE.printStack.isEmpty())
		{
			PrintStream next = INSTANCE.printStack.pop();

			if(hasFlag(INSTANCE.traceMessages, INSTANCE.getFlagName(next)))
			{
				next.println(new Exception().getStackTrace());
			}
			
			next.println(message_);
		}
	}
	
	
	
	
	/**
	 * Enables streams that will print messages.
	 * @param targets_ the flags for requested streams
	 * @since 29-Jul 2006
	 */
	public static void enable(int targets_)
	{
		checkInstance();
		
		INSTANCE.allowedMessages |= targets_;		
	}
	
	
	/**
	 * Disable streams that will print messages.
	 * @param targets_ the flags for requested streams
	 * @since 29-Jul 2006
	 */
	public static void disable(int targets_)
	{
		checkInstance();
		
		targets_ = ~targets_;
		INSTANCE.allowedMessages &= targets_;
	}
	
	
	/**
	 * Sets the streams that will be traced.
	 * Overwrites previous settings.
	 * @param targets_ the streams that should be traced
	 * @since 29-Jul 2006
	 */
	public static void setTrace(int targets_)
	{
		checkInstance();
		
		INSTANCE.traceMessages = targets_;
	}
}